Package: RRPP
Title: Linear Model Evaluation with Randomized Residuals in a
        Permutation Procedure
Version: 2.1.2
Authors@R: c(
    person("Michael", "Collyer", , "mlcollyer@gmail.com", c("aut", "cre"), 
    comment = c(ORCID = "0000-0003-0238-2201")),
    person("Dean", "Adams", , role = "aut", 
    comment = c(ORCID = "0000-0001-9172-7894"))
    )
Description: Linear model calculations are made for many random versions of data.  
    Using residual randomization in a permutation procedure, sums of squares are 
    calculated over many permutations to generate empirical probability distributions 
    for evaluating model effects.  Additionally, coefficients, statistics, fitted values, and residuals generated over many 
    permutations can be used for various procedures including pairwise tests, prediction, classification, and
    model comparison.  This package should provide most tools one could need for the analysis of
    high-dimensional data, especially in ecology and evolutionary biology, but certainly other fields, as well.
Depends: R (>= 4.4.0)
License: GPL (>= 3)
URL: https://github.com/mlcollyer/RRPP
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Imports: parallel, ape, ggplot2, Matrix
Suggests: knitr, rmarkdown, testthat (>= 3.2.0), dplyr, tibble
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-02-18 16:54:13 UTC; m.collyer
Author: Michael Collyer [aut, cre] (<https://orcid.org/0000-0003-0238-2201>),
  Dean Adams [aut] (<https://orcid.org/0000-0001-9172-7894>)
Maintainer: Michael Collyer <mlcollyer@gmail.com>
Repository: CRAN
Date/Publication: 2025-02-18 23:40:12 UTC
Config/testthat/edition: 3
Built: R 4.6.0; ; 2026-04-26 07:20:01 UTC; unix
