# Generated by roxygen2: do not edit by hand
S3method(anova,lm.rrpp)
S3method(anova,measurement.error)
S3method(coef,lm.rrpp)
S3method(fitted,lm.rrpp)
S3method(logLik,lm.rrpp)
S3method(model.frame,lm.rrpp)
S3method(model.matrix,lm.rrpp)
S3method(na.omit,rrpp.data.frame)
S3method(plot,interSubVar)
S3method(plot,kcomp)
S3method(plot,lm.rrpp)
S3method(plot,looCV)
S3method(plot,measurement.error)
S3method(plot,model.comparison)
S3method(plot,ordinate)
S3method(plot,predict.lm.rrpp)
S3method(plot,trajectory.analysis)
S3method(predict,lm.rrpp)
S3method(print,ICCstats)
S3method(print,anova.lm.rrpp)
S3method(print,betaTest)
S3method(print,coef.lm.rrpp)
S3method(print,kcomp)
S3method(print,lm.rrpp)
S3method(print,looCV)
S3method(print,lr_test)
S3method(print,measurement.error)
S3method(print,model.comparison)
S3method(print,ordinate)
S3method(print,pairwise)
S3method(print,pairwise.model.Z)
S3method(print,predict.lm.rrpp)
S3method(print,summary.betaTest)
S3method(print,summary.lm.rrpp)
S3method(print,summary.manova.lm.rrpp)
S3method(print,summary.ordinate)
S3method(print,summary.pairwise)
S3method(print,summary.trajectory.analysis)
S3method(print,trajectory.analysis)
S3method(residuals,lm.rrpp)
S3method(summary,ICCstats)
S3method(summary,anova.lm.rrpp)
S3method(summary,betaTest)
S3method(summary,coef.lm.rrpp)
S3method(summary,kcomp)
S3method(summary,lm.rrpp)
S3method(summary,looCV)
S3method(summary,lr_test)
S3method(summary,manova.lm.rrpp)
S3method(summary,measurement.error)
S3method(summary,model.comparison)
S3method(summary,ordinate)
S3method(summary,pairwise)
S3method(summary,pairwise.model.Z)
S3method(summary,predict.lm.rrpp)
S3method(summary,trajectory.analysis)
S3method(terms,lm.rrpp)
export(ICCstats)
export(QRforX)
export(add.trajectories)
export(add.tree)
export(anova.lm.rrpp)
export(anova.measurement.error)
export(betaTest)
export(classify)
export(convert2ggplot)
export(effect.size)
export(focusMEonSubjects)
export(getANOVAStats)
export(getModelCov)
export(getModels)
export(getPermInfo)
export(getResCov)
export(getTerms)
export(interSubVar)
export(kcomp)
export(lm.rrpp)
export(lm.rrpp.ws)
export(looCV)
export(lr_test)
export(mahal_dist)
export(manova.update)
export(measurement.error)
export(model.comparison)
export(ordinate)
export(pairwise)
export(pairwise.model.Z)
export(plot.kcomp)
export(plot.lm.rrpp)
export(plot.ordinate)
export(plot.predict.lm.rrpp)
export(prep.lda)
export(print.ICCstats)
export(print.anova.lm.rrpp)
export(print.coef.lm.rrpp)
export(print.lm.rrpp)
export(print.model.comparison)
export(print.pairwise)
export(print.pairwise.model.Z)
export(print.predict.lm.rrpp)
export(print.summary.lm.rrpp)
export(print.summary.pairwise)
export(print.summary.trajectory.analysis)
export(print.trajectory.analysis)
export(pval)
export(reveal.model.designs)
export(rrpp.data.frame)
export(scaleCov)
export(summary.ICCstats)
export(summary.anova.lm.rrpp)
export(summary.coef.lm.rrpp)
export(summary.lm.rrpp)
export(summary.model.comparison)
export(summary.pairwise)
export(summary.pairwise.model.Z)
export(summary.predict.lm.rrpp)
export(summary.trajectory.analysis)
export(trajectory.analysis)
export(vec.cor.matrix)
import(Matrix)
import(ggplot2)
import(parallel)
importFrom(ape,collapse.singles)
importFrom(ape,multi2di.phylo)
importFrom(ape,root.phylo)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot.default)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(stats,anova)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,cmdscale)
importFrom(stats,coef)
importFrom(stats,delete.response)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,lm.fit)
importFrom(stats,lm.wfit)
importFrom(stats,loess)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.frame.default)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,optimise)
importFrom(stats,pnorm)
importFrom(stats,prcomp)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,resid)
importFrom(stats,sd)
importFrom(stats,spline)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,object.size)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
