# RRPP
RRPP is a software package for evaluating linear models with residual randomization in a permutation procedure.  S3 Generic used for the lm function can also be used with lm.rrpp, with the chief difference being that lm coefficients, fitted values, and residuals are estimated many times with random permutations of data.

### To install the current RRPP R-package from CRAN:

<i> Within R:</i>

<code> install.packages("RRPP") </code>

### To install the current version of the RRPP R-package from Github using devtools:

<i> Within R:</i>

<code> install.packages("devtools")</code>

<code> devtools::install_github("mlcollyer/RRPP")</code>

The version on github is updated regularly, especially if errors or programming bugs are discovered.
