% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{summary.coef.lm.rrpp}
\alias{summary.coef.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{summary}{coef.lm.rrpp}(object, ...)
}
\arguments{
\item{object}{Object from \code{\link{coef.lm.rrpp}}}

\item{...}{Other arguments passed onto coef.lm.rrpp}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}
