% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.summary.lm.rrpp}
\alias{print.summary.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{summary.lm.rrpp}(x, ...)
}
\arguments{
\item{x}{print/summary object (from \code{\link{summary.lm.rrpp}})}

\item{...}{other arguments passed to print/summary}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}
