% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.data.r
\docType{data}
\name{motionpaths}
\alias{motionpaths}
\title{Simulated motion paths}
\description{
Simulated motion paths
}
\references{
Adams, D. C., and M. L. Collyer. 2009. A general framework for 
the analysis of phenotypic
  trajectories in evolutionary studies. Evolution 63:1143-1154.
}
\author{
Dean Adams
}
\keyword{datasets}
