% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.coef.lm.rrpp}
\alias{print.coef.lm.rrpp}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{coef.lm.rrpp}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{coef.lm.rrpp}}}

\item{...}{Other arguments passed onto coef.lm.rrpp}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}
