% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{plot.looCV}
\alias{plot.looCV}
\title{Plot Function for RRPP}
\usage{
\method{plot}{looCV}(x, axis1 = 1, axis2 = 2, flip = NULL, ...)
}
\arguments{
\item{x}{An object of class \code{\link{looCV}}}

\item{axis1}{A value indicating which component should be 
displayed as the X-axis (default = C1)}

\item{axis2}{A value indicating which component should be 
displayed as the Y-axis (default = C2)}

\item{flip}{An argument that if not NULL can be used to flip 
components in the plot.  
The values need to match axis1 or axis2.  For example, if axis1 = 3 
and axis2 = 4, flip = 1 will not
change either axis; flip = 3 will flip only the horizontal axis; 
flip = c(3, 4) will flip both axes.  Axis will only be flipped in first
plot.}

\item{...}{other arguments passed to plot (helpful to employ
different colors or symbols for different groups).  See}
}
\description{
Plot Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}
\keyword{visualization}
