% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.model.comparison}
\alias{print.model.comparison}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{model.comparison}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{model.comparison}}}

\item{...}{Other arguments passed onto model.comparison}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}
