% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{residuals.lm.rrpp}
\alias{residuals.lm.rrpp}
\title{Extract residuals}
\usage{
\method{residuals}{lm.rrpp}(object, ...)
}
\arguments{
\item{object}{plot object (from \code{\link{lm.rrpp}})}

\item{...}{Arguments passed to other functions}
}
\description{
Extract residuals
}
\examples{
# See examples for lm.rrpp
}
\author{
Michael Collyer
}
\keyword{utilities}
