% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RRPP.utils.r
\name{print.ordinate}
\alias{print.ordinate}
\title{Print/Summary Function for RRPP}
\usage{
\method{print}{ordinate}(x, ...)
}
\arguments{
\item{x}{Object from \code{\link{ordinate}}}

\item{...}{Other arguments passed onto print.ordinate}
}
\description{
Print/Summary Function for RRPP
}
\author{
Michael Collyer
}
\keyword{utilities}
